package epsilon.transfo;

import java.io.File;
import java.net.URI;
import java.util.List;

import org.eclipse.epsilon.common.parse.problem.ParseProblem;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException;
import org.eclipse.epsilon.eol.models.IModel;

public abstract class EpsilonStandalone {

	protected IEolExecutableModule module;

	protected Object result;

	public abstract IEolExecutableModule createModule();

	public abstract List<IModel> getModels() throws Exception;
	
	public abstract URI getSourceFileURI() throws Exception ;

	public void postProcess() {
	};
 
	public void preProcess() {
	};

	public void execute() throws Exception {

		module = createModule();
		module.parse(getSourceFileURI()); // call to the method for using ETL & EVL with validation process
		
		if (module.getParseProblems().size() > 0) {
			System.err.println("Parse errors occured...");
			for (ParseProblem problem : module.getParseProblems()) {
				System.err.println(problem.toString());
			}
			System.exit(-1);
		}

		for (IModel model : getModels()) {
			module.getContext().getModelRepository().addModel(model);
		}

		preProcess();
		result = execute(module);
		System.out.println("result : " + result);
		postProcess();

	}


	protected Object execute(IEolExecutableModule module) throws EolRuntimeException {
		return module.execute();
	}

}

