package epsilon.transfo.etl;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
 
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.epsilon.emc.emf.InMemoryEmfModel;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.dt.ExtensionPointToolNativeTypeDelegate;
import org.eclipse.epsilon.eol.models.IModel;
import org.eclipse.epsilon.etl.EtlModule;

import epsilon.transfo.EpsilonStandalone;

public class EtlStandalone extends EpsilonStandalone {

	private EObject modelIn;
	private Resource resourceOut;
	private String ENS_URI;

	private EtlModule module;

	public EtlStandalone(EObject paramModel, Resource resourceOut, String ENS_URI) throws Exception {
		this.modelIn = paramModel;
		this.resourceOut = resourceOut;
		this.ENS_URI = ENS_URI;
	}

	@Override
	public IEolExecutableModule createModule() {
		module = new EtlModule();
		return module;
	}

	@Override
	public List<IModel> getModels() throws Exception {
		List<IModel> models = new ArrayList<IModel>();

		ResourceSet res_set = new ResourceSetImpl();
		res_set.getResources().add(modelIn.eResource());
		
		InMemoryEmfModel emfModelIn = new InMemoryEmfModel("Param", modelIn.eResource(), ENS_URI);
		emfModelIn.setReadOnLoad(true);
		emfModelIn.setStoredOnDisposal(false);

		models.add(emfModelIn);

		ResourceSet res_set_out = new ResourceSetImpl();
		res_set_out.getResources().add(resourceOut);
		InMemoryEmfModel emfModelOut = new InMemoryEmfModel("Basic", resourceOut, ENS_URI);
		emfModelIn.setReadOnLoad(false);
		emfModelIn.setStoredOnDisposal(true);

		models.add(emfModelOut);

		return models;
	} 

	@Override
	public void preProcess() {
		module.getContext().getNativeTypeDelegates().add(new ExtensionPointToolNativeTypeDelegate()	);
	}

	@Override
	public void postProcess() {

	}

	@Override
	public URI getSourceFileURI() throws Exception {
		URI uri = URI.create("platform:plugin/epsilon.transfo/epsilon/transfo/etl/transfoMinimal.etl"); // for xtext
		
		return uri;
	}

}
